@@ -4,8 +4,11 @@
 package org.gcube.portlets.user.tdtemplate.client.templatecreator.view;
 
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
+import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnData;
 import org.gcube.portlets.user.tdtemplate.client.TdTemplateConstants;
 import org.gcube.portlets.user.tdtemplate.client.TdTemplateController;
 import org.gcube.portlets.user.tdtemplate.client.ZIndexReference;
@@ -15,10 +18,16 @@ import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractRes
 import org.gcube.portlets.user.tdtemplate.client.templatecreator.SetColumnTypeDefinition;
 import org.gcube.portlets.user.tdtemplate.client.templatecreator.SetColumnTypeDialogManager;
 import org.gcube.portlets.user.tdtemplate.client.templatecreator.flow.WindowFlowCreate;
+import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.external.updater.UpdateColumnDataByReference;
 import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.suggestion.ConstraintSuggestionLabel;
 import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.suggestion.HtmlLabel;
 import org.gcube.portlets.user.tdtemplate.client.templatecreator.view.suggestion.SuggestionContainer;
 import org.gcube.portlets.user.tdtemplate.shared.TdFlowModel;
+import org.gcube.portlets.user.tdtemplate.shared.TdTColumnCategory;
+import org.gcube.portlets.user.tdtemplate.shared.TdTDataType;
+import org.gcube.portlets.user.tdtemplate.shared.TdTFormatReference;
+import org.gcube.portlets.user.tdtemplate.shared.TdTTimePeriod;
+import org.gcube.portlets.user.tdtemplate.shared.TemplateExpression;
 
 import com.extjs.gxt.ui.client.Style.ButtonScale;
 import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
@@ -36,6 +45,7 @@ import com.extjs.gxt.ui.client.widget.ContentPanel;
 import com.extjs.gxt.ui.client.widget.HorizontalPanel;
 import com.extjs.gxt.ui.client.widget.Html;
 import com.extjs.gxt.ui.client.widget.LayoutContainer;
+import com.extjs.gxt.ui.client.widget.MessageBox;
 import com.extjs.gxt.ui.client.widget.button.Button;
 import com.extjs.gxt.ui.client.widget.button.SplitButton;
 import com.extjs.gxt.ui.client.widget.layout.BorderLayout;
@@ -77,7 +87,7 @@ public class TemplatePanel {
 	private Button addFlow;
 	private MenuItem createFlow;
 	private MenuItem removeFlow;
-	private SelectionListener listenerAddFlow;
+	private SelectionListener<ButtonEvent> listenerAddFlow;
 	protected ToolBar toolbar;
 	private Button actionsButton;
 	private boolean validateTemplateEventFire = true; //USED BY ADD COLUMN ACTION
@@ -85,6 +95,7 @@ public class TemplatePanel {
 	private boolean isValidTemplate = false;
 	protected ConstraintSuggestionLabel constraintSuggestionLabel;
 	protected ZIndexReference zIndexReference;
+	private SplitButton cloneColumn = new SplitButton("Clone Column");
 	//USED TO CREATE ONLY POST-OPERATIONS
 	/**
 	 * Instantiates a new template panel.
@@ -100,8 +111,6 @@ public class TemplatePanel {
 
 		this.tableContainer.setHeaderVisible(false);
 		this.tableContainer.setBorders(false);
-//		this.tableContainer.setLayout(new FitLayout());
-//		this.tableContainer.setStyleAttribute("margin-top", "50px");
 		inizializeTableTemplate();
 		inizializeToolBar();
 	
@@ -113,16 +122,13 @@ public class TemplatePanel {
 
 		tableContainer.setBodyBorder(false);
 		tableContainer.setBorders(false);
-		
-//		htmlTitleLabel = new HtmlLabel(TdTemplateConstants.DEFINITION_AND_VALIDATION_TEMPLATE, "", "");
+
 		tableContainer.add(htmlTitleLabel);
 		tableContainer.add(suggestionContainer);
 		
 		flexTableContainer.add(flexTableTemplate);
 		tableContainer.add(flexTableContainer);
 
-//		tableContainer.add(cs.getAnchor());
-
 		centralContainer.add(tableContainer);
 		centralContainer.setBorders(false);
 		
@@ -133,7 +139,6 @@ public class TemplatePanel {
 			@Override
 			public void handleEvent(BaseEvent be) {
 				setBorderAsOnError(false);
-				
 			}
 		});
 		
@@ -161,6 +166,7 @@ public class TemplatePanel {
 				GWT.log("Init table");
 				tableContainer.setEnabled(true);
 				initTableColumns(flexTableTemplate, 0, numColumns);
+				updateMenuOfCloneableColumns();
 				TemplatePanel.this.controller.getInternalBus().fireEvent(new SetColumnTypeCompletedEvent());
 			}
 		};
@@ -173,6 +179,13 @@ public class TemplatePanel {
 	
 	
 
+	/**
+	 * @return the templateSwitcherInteface
+	 */
+	public TemplateSwitcherInteface getTemplateSwitcherInteface() {
+		return templateSwitcherInteface;
+	}
+
 	/**
 	 * Creates the boder layout.
 	 */
@@ -190,8 +203,7 @@ public class TemplatePanel {
 	    southData.setCollapsible(true);  
 	    southData.setFloatable(true);  
 	    southData.setMargins(new Margins(5,0,0,0));  
-//	    southData.setMaxSize(250);
-//	    southData.setMinSize(50);
+
 		southContainer = new ContentPanel();
 		southContainer.setHeaderVisible(true);
 		southContainer.setBorders(false);
@@ -199,13 +211,9 @@ public class TemplatePanel {
 		southContainer.setHeading("Filters");
 		southContainer.setScrollMode(Scroll.AUTOY);
 		
-//		centralContainer.setId("CentralContainerTemplatePanel");
-//		tableContainer.setId("TableContainerTemplatePanel");
 		centralContainer.setHeight(TdTemplateConstants.HEIGHT_PRINCIPAL);
 		centralContainer.setScrollMode(Scroll.AUTOY);
-		principalContainer.add(centralContainer, centerData);
-//		principalContainer.add(southContainer, southData);
-		
+		principalContainer.add(centralContainer, centerData);	
 	}
 	
 	/**
@@ -252,7 +260,7 @@ public class TemplatePanel {
 //			table.getCellFormatter().getElement(i, colIndex).getStyle().setOpacity(opacity);
 //		}
 //	}
-//	
+	
 	/**
 	 * Sets the column type opacity.
 	 *
@@ -285,6 +293,7 @@ public class TemplatePanel {
 
 		// Add a button that will add more rows to the table
 	    Button addColumnButton = new Button("Add Column", TdTemplateAbstractResources.columnAdd());
+	    addColumnButton.setTitle("Append a new Column");
 		addColumnButton.setScale(ButtonScale.MEDIUM);
 		addColumnButton.setIconAlign(IconAlign.TOP);
 //		addColumnButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
@@ -293,12 +302,13 @@ public class TemplatePanel {
 			
 			@Override
 			public void componentSelected(ButtonEvent ce) {
-				addColumn(flexTableTemplate);
-				
+				addColumn(flexTableTemplate, true);
+				updateMenuOfCloneableColumns();
 			}
 		});
 
 		Button removeRowButton = new Button("Remove Column", TdTemplateAbstractResources.columnRemove());
+		removeRowButton.setTitle("Remove Last Column");
 		removeRowButton.setScale(ButtonScale.MEDIUM);
 		removeRowButton.setIconAlign(IconAlign.TOP);
 //		removeRowButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
@@ -310,63 +320,33 @@ public class TemplatePanel {
 				if(numColumns>1){
 					removeColumn(flexTableTemplate, numColumns-1);
 					setNumColumns(numColumns-1);
+					updateMenuOfCloneableColumns();
 				}
-				
 			}
 		});
 		
-		/*
-		Button filterBySelection = new Button("Filter by Selection", TdTemplateAbstractResources.filter16());
-		filterBySelection.setScale(ButtonScale.SMALL);
-		filterBySelection.setIconAlign(IconAlign.LEFT);
-		filterBySelection.setToolTip("Filter by Check Column");
+		cloneColumn.setIcon(TdTemplateAbstractResources.columnClone());
+		cloneColumn.setTitle("Create a clone column of an existing column");
+		cloneColumn.setScale(ButtonScale.MEDIUM);
+		cloneColumn.setIconAlign(IconAlign.TOP);
 //		removeRowButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
 		
-		filterBySelection.addSelectionListener(new SelectionListener<ButtonEvent>() {
+		cloneColumn.addSelectionListener(new SelectionListener<ButtonEvent>() {
 
 			@Override
 			public void componentSelected(ButtonEvent ce) {
-
-				List<ColumnElement> columnsSelected = new ArrayList<ColumnElement>();
-				for (ColumnDefinitionView column : columnsDefined) {
-					if(column.isSelected()){
-//						column.setAsReadOnly(true);
-//						setBodyColumnVisible(flexTableTemplate, column.getColumnIndex(), false);
-						columnsSelected.add(column);
-					}
+				if(numColumns<1){
+					MessageBox.info("Info..", "You must define at least one column to create a clone", null);
+					return;
 				}
-				
-				if(columnsSelected.size()>0){
-					MultiFilterByColumns multi;
-					try {
-				
-						multi = new MultiFilterByColumns(columnsSelected);
-//						multi.getWindow().show();
-						LayoutContainer lc = multi.getPanel();
-						lc.setStyleAttribute("margin-bottom", "2px");
-						southContainer.add(lc);
-						southContainer.layout(true);
-					} catch (ColumnNotTypedException e) {
-						MessageBox.info("Set data type!", "You must set the data type for selected column/s before to add a filter", null).show();
-					}
-			
-				}else
-					MessageBox.info("No selection", "You must select (through checkbox) at least one column", null).show();
-				
+				cloneColumn.getMenu().show(cloneColumn);
 			}
-		});*/
-	
+		});
 		
-		toolbar.add(addColumnButton);
-		toolbar.add(removeRowButton);
-		
-		toolbar.add(new FillToolItem());
 		addFlow = new SplitButton("Flow");
 		addFlow.setIcon(TdTemplateAbstractResources.flow24());
-		
 		addFlow.setScale(ButtonScale.MEDIUM);
 		addFlow.setIconAlign(IconAlign.TOP);
-//		addColumnButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
 		addFlow.setMenu(createFlowMenuAndListener());
 		
 		listenerAddFlow = new SelectionListener<ButtonEvent>() {
@@ -379,13 +359,37 @@ public class TemplatePanel {
 		};
 		
 		addFlow.addSelectionListener(listenerAddFlow);
-		
 		enableFlow(false);
-		
+
+		toolbar.add(addColumnButton);
+		toolbar.add(removeRowButton);
+		toolbar.add(cloneColumn);
+		toolbar.add(new FillToolItem());
 		toolbar.add(addFlow);
 		tableContainer.setTopComponent(toolbar);
 	}
 	
+	private void updateMenuOfCloneableColumns(){
+		if(cloneColumn.getMenu()!=null)
+			cloneColumn.getMenu().removeAll();
+		else
+			cloneColumn.setMenu(new Menu());
+		
+		for (final ColumnDefinitionView columnDefinitionView : columnsDefined) {
+			
+			MenuItem item = new MenuItem(columnDefinitionView.getColumnLabel());
+			item.addSelectionListener(new SelectionListener<MenuEvent>() {
+
+				@Override
+				public void componentSelected(MenuEvent ce) {
+					cloneColumn(flexTableTemplate, columnDefinitionView.getColumnIndex(), 1);
+				}
+			});
+			
+			cloneColumn.getMenu().add(item);
+		}
+	}
+
 	/**
 	 * Gets the toolbar.
 	 *
@@ -403,7 +407,7 @@ public class TemplatePanel {
 	public SetColumnTypeDefinition getSetColumnTypeDefinition() {
 		return setColumnTypeDefinition;
 	}
-
+	
 	/**
 	 * Creates the flow menu and listener.
 	 *
@@ -555,11 +559,6 @@ public class TemplatePanel {
 
 			final ColumnDefinitionView columnDef = new ColumnDefinitionView(this, indexOffset, new SetColumnTypeDialogManager(setColumnTypeDefinition.getListCategory(), controller));
 			
-//			flexTable.setWidget(0, indexOffset, columnDef.getColumnHeaderValue());
-			
-//			HorizontalPanel hpHeader = getEditableLabelPanel(columnDef);
-//			flexTable.setWidget(0, indexOffset, hpHeader);
-			
 			EditableLabelColumnDefinitionView editableLabel = new EditableLabelColumnDefinitionView(zIndexReference.getZIndex()+1);
 			HorizontalPanel hpHeader = editableLabel.getEditableLabelPanel(columnDef, zIndexReference);
 			flexTable.setWidget(0, indexOffset, hpHeader);
@@ -616,7 +615,6 @@ public class TemplatePanel {
 	 * @param columnIndex the column index
 	 */
 	public void clearCell(int rowIndex, int columnIndex){
-		
 		try{
 			GWT.log("Clear cell rowIndex, "+rowIndex+", column index "+columnIndex);
 			flexTableTemplate.clearCell(rowIndex, columnIndex);	
@@ -626,16 +624,101 @@ public class TemplatePanel {
 	}
 	
 	
+	
 	/**
-	 * Add a row to the flex table.
+	 * Adds the column.
 	 *
 	 * @param flexTable the flex table
+	 * @param validateTemplate the validate template
 	 */
-	private void addColumn(FlexTable flexTable) {
+	private void addColumn(FlexTable flexTable, boolean validateTemplate) {
 
 		initTableColumns(flexTable, numColumns, 1);
 		setNumColumns(numColumns+1);
-		validateTemplate();
+		if(validateTemplate)
+			validateTemplate();
+	}
+
+
+	/**
+	 * Clone column.
+	 *
+	 * @param indexSourceColumn the index source column
+	 * @param numberOfClones the number of clones
+	 */
+	private void cloneColumn(FlexTable flexTable, int indexSourceColumn, int numberOfClones) {
+
+		//RANGE VALIDATION
+		if(indexSourceColumn>=0 && indexSourceColumn<columnsDefined.size()){
+			ColumnDefinitionView source = columnsDefined.get(indexSourceColumn);
+			
+			for (int i=0; i<numberOfClones;i++) {
+				addColumn(flexTable, false);
+				ColumnDefinitionView clone = columnsDefined.get(numColumns-1); //get current latest column
+				copyColumnFields(source, clone);
+			}
+			validateTemplate();
+		}	
+	}
+	
+	private void copyColumnFields(ColumnDefinitionView source, ColumnDefinitionView destination){
+		
+		if(source.getColumnName()!=null && !source.getColumnName().isEmpty())
+			destination.setColumnHeaderValue("Clone "+destination.getColumnName());
+		else
+			destination.setColumnHeaderValue("A Cloned Column");
+		
+		//SOURCE FIELDS
+		TdTColumnCategory sTdCategory = source.getSelectedColumnCategory();
+		TdTDataType sTdDataType = source.getSelectedDataType();
+		ColumnData sColumnData = source.getReferenceColumnData();
+		
+		if(sTdCategory==null)
+			return;
+		
+		//SET VALUE HERE
+		SetColumnTypeDialogManager dColTypeDialogMng = destination.getSetColumnTypeViewManager();
+
+		destination.setSpecialCategoryType(source.getSpecialCategoryType()); //SETTING SPECIAL CATEGORY
+		dColTypeDialogMng.getScbCategory().setSimpleValue(sTdCategory.getName());
+		
+		if(sTdDataType!=null)
+			dColTypeDialogMng.getScbDataType().setValue(sTdDataType);
+		
+		//HAS IT A FORMAT REFERENCE?
+		if(sTdDataType.getFormatReferenceIndexer()!=null){
+			TdTFormatReference format = source.getSelectedDataTypeFormat();
+			GWT.log("FORMAT: "+format);
+			dColTypeDialogMng.getScbDataTypeFormat().setValue(format);
+		}
+		
+		//IS A DIMENSION?
+		if(sColumnData!=null){
+			String tabularResourceName = source.getSetColumnTypeDialogManager().getReferenceTabularResourceName()!=null?source.getSetColumnTypeDialogManager().getReferenceTabularResourceName():"";
+			dColTypeDialogMng.setTabularResourceName(tabularResourceName);
+			new UpdateColumnDataByReference(dColTypeDialogMng, sColumnData);
+		}
+		
+//		if(source.getSelectedLocale()!=null){
+//			colTypeDialogMng.setSelectedLocale(destination.getSelectedLocale(), true);
+//		}
+		
+		//IS A TIMEDIMENSION?
+		if(source.getTimePeriod()!=null){
+			Map<String, String> valueFormats = new HashMap<String, String>();
+			valueFormats.put(source.getTimePeriodFormat().getId(), ""); //the formatIdentifier
+			
+			GWT.log("Value Format : "+source.getTimePeriodFormat() + ", to Time Period: "+source.getTimePeriod());
+			dColTypeDialogMng.setSelectTimePeriod(new TdTTimePeriod(source.getTimePeriod(),valueFormats));
+		}
+		
+		//HAS IT A RULE?
+		if(source.getRulesExpressions()!=null){
+			for (TemplateExpression expres : source.getRulesExpressions()) {
+				destination.addRule(expres, false, true);
+			}
+		}
+		
 	}
 	
 
@@ -648,17 +731,6 @@ public class TemplatePanel {
 		this.numColumns = numColumns;
 		
 	}
-
-
-//	private void addHeaderColumns(FlexTable flexTable) {
-//		int numRows = flexTable.getRowCount();
-//		
-//		int numColumns = flexTable.getCellCount(numRows);
-//		
-//		for (int i = 0; i < numColumns; i++) {
-//			flexTable.setWidget(1, i, new Label("Column "+i+1));
-//		}
-//	}
 	
 	/**
  * Remove a row from the flex table.
@@ -667,10 +739,7 @@ public class TemplatePanel {
  * @param columnIndex the column index
  */
 	private void removeColumn(FlexTable flexTable, int columnIndex) {
-
-	
 		GWT.log("Remove column at index: "+columnIndex);
-		
 		if (columnIndex > 0) {
 			int numRows = flexTable.getRowCount();
 			GWT.log("Current Table row size: "+numRows);
@@ -750,8 +819,6 @@ public class TemplatePanel {
 					controller.getInternalBus().fireEvent(new TemplateCompletedEvent(false));
 				
 				enableFlow(false);
-//				enableFilter(false);
-//				resetFilterPanel();
 				break;
 			}
 		}
@@ -765,15 +832,6 @@ public class TemplatePanel {
 				controller.getInternalBus().fireEvent(new TemplateCompletedEvent(true));
 				
 			enableFlow(true);
-			/*enableFilter(true);
-			enanbleColumnTypes(false);
-			BaloonPanel baloonPanel = new BaloonPanel("Do you want add filters?", true);
-			int zIndex = controller.getWindowZIndex();
-			int zi = zIndex+1;
-			baloonPanel.getElement().getStyle().setZIndex(zi);
-			baloonPanel.showRelativeTo(southContainer);
-			*/
-//			setCheckColumnsEnabled(true);
 		}
 	}
 
@@ -855,16 +913,6 @@ public class TemplatePanel {
 		return controller;
 	}
 
-//	/**
-//	 * Enable panel.
-//	 *
-//	 * @param b the b
-//	 */
-//	public void enablePanel(boolean b) {
-//		principalContainer.setEnabled(b);
-//		
-//	}
-
 	/**
 	 * Sets the visible suggests.
 	 *
@@ -925,10 +973,12 @@ public class TemplatePanel {
 	}
 	
 	/**
- * @param b
- */
+	 * Sets the editable header value.
+	 *
+	 * @param columnIndex the column index
+	 * @param b the b
+	 */
 	public void setEditableHeaderValue(int columnIndex, boolean b) {
 		columnsDefined.get(columnIndex).getEdiTableLabel().setEditableVisible(b);
 	}
-	
 }
